﻿using ImageProcessingGraph.Editor.Nodes.NodeAttributes;
using Unity.Burst;
using Unity.Collections;
using Unity.Jobs;
using UnityEngine;

namespace ImageProcessingGraph.Editor.Nodes.Fun_Nodes.Texture
{
    [NodeInfoAttribute("Invert Channel", "Adjustments/InvertChannel", true)]
    public class ChannelInvertNode : BaseImageNode
    {
        [NodeAttributes.Input("Input Channel")]
        public SplitChannelData inputChannel;

        [NodeAttributes.Output("Output Channel")]
        public SplitChannelData outputChannel;

        public override void Process()
        {
            NativeArray<byte> input = new NativeArray<byte>(inputChannel.ChannelData, Allocator.Persistent);
            NativeArray<byte> output = new NativeArray<byte>(inputChannel.ChannelData.Length, Allocator.Persistent);

            InvertChannelJob job = new InvertChannelJob
            {
                input = input,
                output = output
            };

            job.Run();

            byte[] outputArray = output.ToArray();

            outputChannel = new SplitChannelData(outputArray, (inputChannel.Width, inputChannel.Height));
        }
    }

    [BurstCompile]
    public struct InvertChannelJob : IJob
    {
        [ReadOnly] public NativeArray<byte> input;
        [WriteOnly] public NativeArray<byte> output;

        public void Execute()
        {
            int length = input.Length;
            for (int i = 0; i < length; i++)
            {
                output[i] = (byte)(255 - input[i]);
            }
        }
    }
}